/* $Id: init.c,v 1.1 2000/12/10 11:31:36 marcus Exp $
******************************************************************************

   GGIGL extension initialization

   Copyright (C) 1998      Andreas Beck		[becka@ggi-project.org]
   Copyright (C) 1999-2000 Marcus Sundberg	[marcus@ggi-project.org]
  
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <string.h>

#include "config.h"
#include <ggi/internal/ggigl.h>

#include <ggi/gg.h> /* We use LibGG to manage config files */
#include <ggi/ggigl.h>


/* Static variables 
 */
static int 	 _ggiglLibIsUp       = 0;
static void	*_ggiglConfigHandle  = NULL;
static char      _ggiglconfstub[512] = GGIGLCONFFILE;
static char     *_ggiglconffile      = _ggiglconfstub+GGIGLTAGLEN;

/* Extension ID. Defaulting to -1 should make segfault on abuse more likely...
   This is exported so that sublibs can use it.
 */
EXPORTVAR ggi_extid _ggiglID = -1;


static int
dummyfunc(void)
{
	return -1;
}

/* Reset all function pointers to dummyfunc */
static void
clearfuncs(ggiglpriv *priv)
{
	priv->swapbuffers = (void *) dummyfunc; 
}


/* This function is called by LibGGI when the API-list changes */
static int
changed(ggi_visual_t vis, int whatchanged)
{
	switch (whatchanged) {
	case GGI_CHG_APILIST: {
		int i;
		const char *fname;
		ggi_dlhandle *lib;
		char api[GGI_MAX_APILEN], args[GGI_MAX_APILEN];

		clearfuncs(GGIGLEXT_PRIV(vis));

		for (i=0; ggiGetAPI(vis, i, api, args) == 0; i++) {
			strcat(api,"-ggigl");
			fname = ggMatchConfig(_ggiglConfigHandle, api, NULL);
			if (fname == NULL) {
				/* No special implementation for this sublib */
				continue;
			}
			lib = ggiExtensionLoadDL(vis, fname, args, NULL,
						 "GGIGLdl_");
		}
	}
	break;
	}
                                
	return 0;
}


/* Initialize the extension
 */
int
ggiglInit(void)
{
	int err;

	_ggiglLibIsUp++;
	if (_ggiglLibIsUp > 1) return 0; /* Initialize only at first call. */

	err = ggLoadConfig(_ggiglconffile, &_ggiglConfigHandle);
	if (err != GGI_OK) {
		fprintf(stderr,"LibGGIGL: couldn't open %s\n",
			_ggiglconffile);
		_ggiglLibIsUp--;
		return err;
	}

	_ggiglID = ggiExtensionRegister("GGIGL", sizeof(ggiglpriv), changed);

	if (_ggiglID < 0) {
		fprintf(stderr,
			"LibGGIGL: failed to register as extension.\n");
		_ggiglLibIsUp--;
		ggFreeConfig(_ggiglConfigHandle);
		return _ggiglID;
	}

	return 0;
}


/* Deinitialize the extension
 */
int
ggiglExit(void)
{
	int rc;

	if (!_ggiglLibIsUp) return -1;

	if (_ggiglLibIsUp > 1) {
		/* Exit only at last call */
		_ggiglLibIsUp--;
		return 0;
	}

	rc = ggiExtensionUnregister(_ggiglID);
	ggFreeConfig(_ggiglConfigHandle);

	_ggiglLibIsUp = 0;

	return rc;
}


/* Attach the extension to a visual
 */
int
ggiglAttach(ggi_visual_t vis, int *attribs)
{
	int rc;

	rc = ggiExtensionAttach(vis, _ggiglID);
	if (rc == 0) {
		/* We are actually creating the primary instance. */
		memset(GGIGLEXT_PRIV(vis), 0, sizeof(ggiglpriv));
		GGIGLEXT_PRIV(vis)->priv = NULL;

		/* Now fake an "API change" so the right libs get loaded */
		changed(vis, GGI_CHG_APILIST);
	}

	/* Set attribs */
	ggiglSetAttribs(vis, attribs);

	return rc;
}
